import os
import re

# Always use the directory where this script is located
SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
JAVA_ENUM_DIR = SCRIPT_DIR
OUTPUT_TS_FILE = os.path.join(SCRIPT_DIR, 'ESIEnum.generated.ts')

def parse_java_enum(java_file_path):
    with open(java_file_path, encoding='utf-8') as f:
        content = f.read()

    # Extract enum name
    enum_name_match = re.search(r'enum\s+(\w+)', content)
    if not enum_name_match:
        return None
    enum_name = enum_name_match.group(1)

    # Extract enum constants
    constants = []
    for m in re.finditer(r'(\w+)\s*\(\s*([\d]+)\s*,\s*"([^"]+)"', content):
        const_name, value, message = m.groups()
        th, en = message, ""
        if '(' in message and ')' in message:
            th = message.split('(')[0].strip()
            en = message.split('(')[1].replace(')', '').strip()
        constants.append({
            'name': const_name,
            'value': int(value),
            'th': th,
            'en': en if en else th
        })
    return enum_name, constants

def to_camel_case(s):
    # Converts SNAKE_CASE or UPPER_CASE to PascalCase
    parts = s.lower().split('_')
    return ''.join([p.capitalize() for p in parts if p])

def java_enum_to_ts(enum_name, constants):
    ts_enum_name = enum_name
    if ts_enum_name.startswith('TYPE_'):
        ts_enum_name = ts_enum_name[5:]
    ts_enum_name = ''.join([w.capitalize() for w in ts_enum_name.lower().split('_')])
    if ts_enum_name.endswith('status'):
        ts_enum_name = ts_enum_name[:-6] + 'Status'
    ts = f'    {ts_enum_name}: import("./enumsClass").Enums<{{\n'
    for c in constants:
        key = c['name']
        # Remove common prefixes
        for prefix in ['STATUS_', 'TYPE_']:
            if key.startswith(prefix):
                key = key[len(prefix):]
        key = to_camel_case(key)
        ts += f'        readonly {key}: {{\n'
        ts += f'            readonly value: {c["value"]};\n'
        ts += f'            readonly th: "{c["th"]}";\n'
        ts += f'            readonly en: "{c["en"]}";\n'
        ts += f'        }};\n'
    ts += f'    }}>;\n'
    return ts_enum_name, ts

def main():
    enum_blocks = []
    for fname in os.listdir(JAVA_ENUM_DIR):
        if fname.startswith('TYPE_') and fname.endswith('.java'):
            result = parse_java_enum(os.path.join(JAVA_ENUM_DIR, fname))
            if result:
                enum_name, constants = result
                _, ts_block = java_enum_to_ts(enum_name, constants)
                enum_blocks.append(ts_block)

    with open(OUTPUT_TS_FILE, 'w', encoding='utf-8') as f:
        f.write('// Auto-generated from Java enums\n')
        f.write('declare const ESIEnum: {\n')
        for block in enum_blocks:
            f.write(block)
        f.write('}\n')

if __name__ == '__main__':
    main()